/*

Convdb : A flexible format text to "Palm" datebook database converter.
Copyright (c) 2000 Narayan Natarajan

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

-----------------------------------------------
RCS stuff:
-----------------------------------------------


Author : Narayan Natarajan AKA $Author: venkman $
Date   : $Date: 2000/02/17 01:11:38 $
Version: $Revision: 2.7 $

Convdb.c : Flexible format text to datebook database converter.



*/

#include "values.h"
#include "strnicmp.h"

Datebook dbook;
Datebook init;
extern Recdef recorddef[100];

/*extern int getopt(int argc, char * const argv[], const char *optstring);
*/
extern char *optarg;
extern int optind, opterr, optopt;

extern void parserec();
extern void parse_field();
extern int fmtcnt;
extern int multiline;
long alarmtype, alarmadv, alarmset;
int dynamem=MEM_CHUNK;
void initialize();
void read_gendb();
void print_dbook();
int timeadjust=0;


main(argc, argv)
char **argv;
int argc;
{
	FILE *fp;
	char options;


	alarmset = 0;

	options = getopt(argc, argv, "xh:m:d:");

	switch(options)
	{
		case 'm':
			alarmtype=0;
			alarmadv=atoi(optarg);
			alarmset=1;
			break;
		case 'h':
			alarmtype=1;
			alarmadv=atoi(optarg);
			alarmset=1;
			break;
		case 'd':
			alarmtype=2;
			alarmadv=atoi(optarg);
			alarmset=1;
		case 'x':
			fprintf(stdout, "Usage: %s -[OPTIONS <value>] <filename>\n",
					argv[0]);
			fprintf(stdout, "\tAlarm may be set globally:\n");
			fprintf(stdout, "\t-m N     N minutes in advance\n");
			fprintf(stdout, "\t-h N     N hours   in advance\n");
			fprintf(stdout, "\t-d N     N days    in advance\n");
			fprintf(stdout, "\t-x  e'x'plain more : Help    \n");
			fprintf(stdout, "$Revision: 2.7 $\n");
			fprintf(stdout, "\n");
			fprintf(stdout, "    +----------------------------------+\n");
			fprintf(stdout, "    |                                  |\n");
			fprintf(stdout, "    |  Author: Narayan Natarajan       |\n");
			fprintf(stdout, "    |  E-mail: venkman1@ix.netcom.com  |\n");
			fprintf(stdout, "    |                                  |\n");
			fprintf(stdout, "    +----------------------------------+\n");
			fprintf(stdout, "\n");
			fprintf(stdout, "Field definitions within the file \n");
			fprintf(stdout, "\n");
			fprintf(stdout, "   %%d   [d]ate \n");
			fprintf(stdout, "   %%y   [y]ear \n");
			fprintf(stdout, "   %%m   [m]onth \n");
			fprintf(stdout, "   %%h   [h]our: start hour\n");
			fprintf(stdout, "   %%i    i    : start min (i follows h)\n");
			fprintf(stdout, "   %%e   [e]nd : end hour \n");
			fprintf(stdout, "   %%f    f    : end min (f follows e)\n");
			fprintf(stdout, "   %%t   [t]ime span: duration of event\n");
			fprintf(stdout, "   %%r   [r]epeat flag\n");
			fprintf(stdout, "   %%a   [a]m/pm start time fmt\n");
			fprintf(stdout, "   %%b    b      end time fmt (b follows a)\n");
			fprintf(stdout, "   %%p   [p]rivate flag       \n");
			fprintf(stdout, "   %%u   [u]ntimed event flag \n");
			fprintf(stdout, "   %%v   e[v]ent  \n");
			fprintf(stdout, "   %%n   [n]ote , this is NOT a field!!! \n");
			fprintf(stdout, "\n");
			fprintf(stdout, "NOTE: Note is to be entered in separate lines,\n");
			fprintf(stdout, "followed by a single line with a DOT (.) all\n");
			fprintf(stdout, "by itself to end the note\n");
			fprintf(stdout, "\n");
			fprintf(stdout, "For example:\n");
			fprintf(stdout, "---------\n");
			fprintf(stdout, "%%m	%%d	%%h	%%v	%%n\n");
			fprintf(stdout, "3	1	8	give someone a lot of money\n");
			fprintf(stdout, "me mee  meee \n");
			fprintf(stdout, "oh and me too and me and me\n");
			fprintf(stdout, ".\n");
			fprintf(stdout, "---------\n");
			/* should I include this? */
			/*
			fprintf(stdout, "   NOTE: %%n must be the last field\n");
			*/
			fprintf(stdout, "   Minimum fields required:\n");
			fprintf(stdout, "   Month, date, start_hour, event\n");
			fprintf(stdout, "\n");
			fprintf(stdout, "   Example:\n");
			fprintf(stdout, "   To convert a text file with the following info \n");
			fprintf(stdout, "\n");
			fprintf(stdout, "   3/1/99 8:00 AM Greet myself Happy Birthday!\n");
			fprintf(stdout, "   3/2/98 9:00 AM Ask Boss for a raise\n");
			fprintf(stdout, "\n");
			fprintf(stdout, "   Add this record as the first\n");
			fprintf(stdout, "\n");
			fprintf(stdout, "   %%m/%%d/%%y %%h:%%i %%a %%v\n");
			fprintf(stdout, "\n");


			exit(0);

	}
/*
	printf("Type: %d time %d optind %d\n", alarmtype, alarmadv, optind);
*/

	if ( (argc - optind) > 0 )
	{
		if ( (fp=fopen(argv[optind], "rb" ) ) == NULL )
		{
			fprintf(stderr,"ERROR: %s not found\n", argv[optind]);
			exit(1);
		}
	}
	else
	{
		fprintf(stderr, "Usage: %s -[OPTIONS <value>] <filename>\n",argv[0]);
		fprintf(stderr, "\tAlarm may be set globally:\n");
		fprintf(stderr, "\t-m N     N minutes in advance\n");
		fprintf(stderr, "\t-h N     N hours   in advance\n");
		fprintf(stderr, "\t-d N     N days    in advance\n");
		fprintf(stderr, "\t-x  e'x'plain more : Help    \n");
		fprintf(stderr, "$Revision: 2.7 $\n");
		exit(1);
	}
	initialize();
	read_gendb(fp);
	print_dbook();
}

void read_gendb(fp)
FILE *fp;
{
	char strbuf[BUFSIZ], input[BUFSIZ], output[BUFSIZ], notebuf[5000],
		note[4096];
	char fields[4][BUFSIZ];
	char *ptr;
	int i, temp, diffhr, diffmin, temphr, tempmin;
	int counter=0, space;
	time_t t_start, t_end;
	struct tm starttime, endtime;
	char repeat_type[10];
	struct tm *temptime;
	char tempstr[100];
	time_t current_time,  gm_t_time, lcl_t_time ;
	struct tm  *gm_time, *lcl_time;


/*
this is the weirdest thing I have seen

As per the man pages this is what is listed:

function    Input   Conversion?   Output
-----------------------------------------
time        none    current time  time_t
                    faithful

ctime       time_t  may conv. to  formatted str
                    local time    Wed Nov....

asctime     tm *    none          formats time
                                  Wed Nov...

localtime   time_t  conv time_t   tm *
                    to local time
                    then to tm*

mktime      tm *    assumes local time_t
                    time -->
                    arithmetic

gmtime      time_t  assumes clock tm *
                    is localtime

-------------------------------------------


*/

/* lets see, start with initializing the dbook with 100 entries
we'll see how it goes and take it from there */
if (( dbook.datebook = (Datebook_entry *)
	malloc( (size_t) sizeof(Datebook_entry) * dynamem )) == NULL)
{
	fprintf(stderr,"I Can't get more Power Keptin! We're Shcrewed!\n");
	fprintf(stderr,"Maalox won't work\n");
	exit();
}



		time( &current_time );
		gm_time = gmtime( &current_time );
		gm_t_time = mktime ( gm_time);
#ifdef DEBUG
		printf("\tctime %s\n", ctime(&current_time) );
		printf("GMT is :\n");
		printf("\tgmt asc %s\n", asctime( gm_time ) );
		printf("\tgmt ctm : %s\n", ctime ( &gm_t_time) );
		lcl_time= localtime(&current_time);
		lcl_t_time=mktime(lcl_time);
		printf("\tlcl asc %s\n", asctime( lcl_time ) );
		printf("\tlcl ctm : %s\n", ctime ( &lcl_t_time) );
		printf("Difference : %d\n", current_time - lcl_t_time );
		timeadjust=current_time - lcl_t_time ;
		puts("Testing some dates and stuff");

		gm_time->tm_wday= gm_time->tm_yday= 0;
		gm_t_time = mktime ( gm_time);
		printf("\tctm mkt gmt: %s\n", ctime ( &gm_t_time) );


#endif /* DEBUG */

	/* the first thing is to parse the first record and find out
	how the records are formatted */
	parse_fields(fp);


	while(!feof(fp) )
	{
		if (NULL == fgets(strbuf,BUFSIZ, fp))
			break;

		/* try to see if even one char is not space,
		if true then parse it. O/w skipline.
		*/
		space=1;
		for(i=0; strbuf[i] ; i++)
		{
			if ( !isspace(strbuf[i]) )
			{
				space=0;
				break;
			}
		}
		if ( space )
		{
#ifdef DEBUG
				puts("skipping blank");
				puts(strbuf);
#endif /* DEBUG */
				continue;
		}

#ifdef DEBUG
		puts("line passed");

#endif /* DEBUG */
/* also skip comment */
		if ( strbuf[0] == '#' )
			continue;
#ifdef DEBUG
		else
			puts(strbuf);
#endif /*DEBUG*/

/* all the above garbage proves that it is a bonafide record 
next thing to do is find out if the 'multiline' variable is set,
if set then the note data needs to be read
a fullstop at the beginning of the line by itself defines end of the
record.
*/
		notebuf[0]='\0';
		note[0]='\0';
		if ( multiline )
		while ( 1 )
		{
			fgets(notebuf, 4096, fp );
#ifdef DEBUG
			printf("note line len is %d\n", strlen(notebuf));
#endif /*DEBUG*/
			if ( notebuf[0] == '.' /* &&  strlen(notebuf) == 2 */ )
			{
				/*
				remove the carriage return/newline which is 2 chars
				at the end of the note.
				blah blah <0d> <0a>
				*/
				note[strlen(note) - 2] = '\0';
				break;
			}
			strcat(note, notebuf);
#ifdef DEBUG
			printf("--- %d\n", strlen(note));
#endif /*DEBUG*/
		}
		/* copy note into datebook record */
		strcpy( dbook.datebook[counter].note.string, note);
#ifdef DEBUG
		printf("len of Note :\n%d\n", strlen( note ));
#endif /*DEBUG*/
		dbook.datebook[counter].note.byte = strlen(note);


/* expecting fields */
/*

	year month date sthr stmin endhr endmin repeat	description
*/

		/* send it to be parsed so that required fields appear
		seperated by tabs */

		parserec(strbuf,recorddef,output, &dbook.datebook[counter] );
		strcpy( strbuf , output );

		counter++;
		/* if we get to equal to or greater than (shouldn't happen) then
		bump the memory by another 50 */
		if ( counter >= (dynamem-1) )
		{
			dynamem += MEM_CHUNK;
#ifdef DEBUG
			printf("dynamic memory increased to %d %d\n", dynamem,counter);
#endif /* DEBUG */
			if (( dbook.datebook = (Datebook_entry *)
				realloc(dbook.datebook,
					(size_t) sizeof(Datebook_entry) * dynamem )) == NULL)
			{
				fprintf(stderr,"We're Shcrewed!\n");
				fprintf(stderr,"Realox Maalox won't work\n");
				exit();
			}
#ifdef DEBUG
			puts("Testing memory");
			printf("first entry\n %s\n",dbook.datebook[0].description.string);
#endif /* DEBUG */
		}

	}
	dbook.nrec = counter * 15;
	fprintf(stderr, "%d records\n", counter );
}

void print_dbook()
{
	int i;
	int slong;
	char temp;
	Datebook_entry *datebook, *std_values;
	FILE *fp;

#ifdef DEBUG
	puts("Printing Datebook database file");
#endif /* DEBUG */
	if ( (fp=fopen("output.dba", "wb" ) ) == NULL )
	{
		puts("error opening output.dba");
		exit(1);
	}

	strcpy(dbook.GHeader.string, "output.dba");
	dbook.GHeader.byte = strlen(dbook.GHeader.string);
	/* day view is "100 16" */
	strcpy(dbook.SHeader.string, "100 16");
	dbook.SHeader.byte = strlen(dbook.SHeader.string);

	slong=sizeof(long);

	fwrite(	&init.discard, slong, 1, fp );

		if (dbook.GHeader.byte > 255 )
		{
			fputc(255,fp);
			fwrite( &dbook.GHeader.byte, sizeof(char), 2, fp);
		}
		else
		{
			temp=(char)dbook.GHeader.byte;
			fwrite( &temp, sizeof(char), 1, fp);
		}

		if ( dbook.GHeader.byte != 0 )
		fwrite(     &dbook.GHeader.string, sizeof(char),
			strlen( dbook.GHeader.string ), fp );

		if (dbook.SHeader.byte > 255 )
		{
			fputc(255,fp);
			fwrite( &dbook.SHeader.byte, sizeof(char), 2, fp);
		}
		else
		{
			temp=(char)dbook.SHeader.byte;
			fwrite( &temp, sizeof(char), 1, fp);
		}

		if ( dbook.SHeader.byte != 0 )
		fwrite(     dbook.SHeader.string, sizeof(char),
			strlen( dbook.SHeader.string ), fp );

	fwrite(	&init.category.unknown[0], slong, 1, fp);
	fwrite( &init.category.noc, slong, 1, fp);
	fwrite(	&init.category.lindex, slong, 1, fp);
	fwrite(	&init.category.lid, slong, 1, fp);
	fwrite(	&init.category.lname, slong, 1, fp);

	for ( i = 1 ; i < 11 ; i++ )
	fwrite(	&init.category.unknown[i], slong, 1, fp);


	fwrite(	&dbook.nrec, slong, 1, fp);
#ifdef DEBUG
	printf("Counter %d\n", dbook.nrec );
#endif /* DEBUG */
	std_values=&init.datebook[0];
	for ( i = 0 ; i < dbook.nrec/15 ; i ++ )
	{
		datebook=&dbook.datebook[i];

		fwrite(		&std_values->field[0], slong, 1, fp);
		fwrite(		&std_values->lrec_id, slong, 1, fp);
		fwrite(		&std_values->field[1], slong, 1, fp);
		fwrite(		&std_values->lstat, slong, 1, fp);
		fwrite(		&std_values->field[2], slong, 1, fp);
		fwrite(		&std_values->lpos, slong, 1, fp);
		fwrite(		&std_values->field[3], slong, 1, fp);
		fwrite( 	&datebook->start_time, slong, 1, fp);
		fwrite(		&std_values->field[4], slong, 1, fp);
		fwrite( 	&datebook->end_time, slong, 1, fp);
		fwrite(		&std_values->field[5], slong, 1, fp);
		fwrite(		&std_values->field[6], slong, 1, fp);

		if (datebook->description.byte > 255 )
		{
			fputc(255,fp);
			fwrite( &datebook->description.byte, sizeof(char), 2, fp);
		}
		else
		{
			temp=(char)datebook->description.byte;
			fwrite( &temp, sizeof(char), 1, fp);
		}

		if ( datebook->description.byte != 0 )
		fwrite(     datebook->description.string, sizeof(char),
			strlen( datebook->description.string ), fp );
#ifdef DEBUG
		printf("datebook->description.string %s\n",datebook->description.string);
#endif /* DEBUG */

		fwrite(		&std_values->field[7], slong, 1, fp);
		fwrite(		&std_values->lduration, slong, 1, fp);
		fwrite(		&std_values->field[8], slong, 1, fp);
		fwrite(		&std_values->field[9], slong, 1, fp);

		if (datebook->note.byte > 255 )
		{
			fputc(255,fp);
			fwrite( 	&datebook->note.byte, sizeof(char), 2, fp);
		}
		else
		{
			temp=(char)datebook->note.byte;
			fwrite( &temp, sizeof(char), 1, fp);
		}

		if ( datebook->note.byte != 0 )
		fwrite(     datebook->note.string, sizeof(char),
			strlen( datebook->note.string ), fp );
#ifdef DEBUG
		printf("datebook->note.string %s\n",datebook->note.string);
#endif /* DEBUG */


		fwrite(		&std_values->field[10], slong, 1, fp);
/*		fwrite(		&std_values->luntimed, slong, 1, fp); */
		fwrite(		&datebook->luntimed, slong, 1, fp);
		fwrite(		&std_values->field[11], slong, 1, fp);
/*		fwrite(		&std_values->lprivate, slong, 1, fp); */
		fwrite(		&datebook->lprivate, slong, 1, fp);
		/* testing private flag */
		fwrite(		&std_values->field[12], slong, 1, fp);
		fwrite(		&std_values->lcategory, slong, 1, fp);
		fwrite(		&std_values->field[13], slong, 1, fp);

		if( alarmset == 1 )
		{
			fwrite(		&alarmset, slong, 1, fp);
			fwrite(		&std_values->field[14], slong, 1, fp);
			fwrite(		&alarmadv, slong, 1, fp);
			fwrite(		&std_values->field[15], slong, 1, fp);
			fwrite(		&alarmtype, slong, 1, fp);
			fwrite(		&std_values->field[16], slong, 1, fp);
		}
		else
		{
			fwrite(		&std_values->lalarmset, slong, 1, fp);
			fwrite(		&std_values->field[14], slong, 1, fp);
			fwrite(		&std_values->lalarmadv, slong, 1, fp);
			fwrite(		&std_values->field[15], slong, 1, fp);
			fwrite(		&std_values->lalarmadvtype, slong, 1, fp);
			fwrite(		&std_values->field[16], slong, 1, fp);
		}
#ifdef DEBUG
		printf("reptype %d\n", datebook->repeat.lreptype );
#endif /* DEBUG */
		if ( datebook->repeat.lrepeat == 0 )
		fwrite(		&datebook->repeat.lrepeat, slong, 1, fp);
		else
		{
#ifdef DEBUG
			puts("writing repeat record");
#endif /* DEBUG */
			fwrite( 	&datebook->repeat.lrepeat, slong, 1, fp);
			fwrite( 	&datebook->repeat.lunknown, slong, 1, fp);
			fwrite(     &datebook->repeat.creptype, sizeof(char),
				strlen( datebook->repeat.creptype ), fp );
			fwrite( 	&datebook->repeat.lreptype, slong, 1, fp);
			fwrite( 	&datebook->repeat.lfreq, slong, 1, fp);
			fwrite( 	&datebook->repeat.lenddate, slong, 1, fp);
			fwrite( 	&datebook->repeat.lcustom[0], slong, 1, fp);
			fwrite( 	&datebook->repeat.lcustom[1], slong, 1, fp);
			fwrite( 	&datebook->repeat.lcustom[2], slong, 1, fp);
		}



/*		printf("lrepeat %d\n",std_values->lrepeat ); */
	}
	fclose(fp);
}

void initialize()
{
	Datebook_entry *datebook;
#ifdef DEBUG
	fprintf(stderr, "Initialize\n");
#endif /* DEBUG */
	init.discard=1145176320;
	init.category.unknown[0]=128;
	init.category.noc=0;
	init.category.lindex=54;
	init.category.lid=15;
	init.category.lname=0;
	init.category.unknown[1]=1;
	init.category.unknown[2]=2;
	init.category.unknown[3]=65551;
	init.category.unknown[4]=65537;
	init.category.unknown[5]=65539;
	init.category.unknown[6]=65541;
	init.category.unknown[7]=393221;
	init.category.unknown[8]=65542;
	init.category.unknown[9]=65542;
	init.category.unknown[10]=524289;
	init.nrec=0;

	/* lets try maalox! creating one record */
	if (( init.datebook = (Datebook_entry *) malloc( (size_t) sizeof(Datebook_entry) * 1 )) == NULL)
	{
		fprintf(stderr,"We're Shcrewed!\n");
		fprintf(stderr,"Maalox won't work\n");
		exit();
	}

	/* just for shorter var name */
	datebook=init.datebook;

	datebook->lrec_id = 0;
	/* changing this to 1 to match datebook.dat .. dunno what it does..*/
	datebook->lstat = 1;
	datebook->lpos = 0;
	datebook->lduration = 0;
	datebook->lcategory = 0;

	datebook->field[0]=1;
	datebook->field[1]=1;
	datebook->field[2]=1;
	datebook->field[3]=3;
	datebook->field[4]=1;
	datebook->field[5]=5;
	datebook->field[6]=0;
	datebook->field[7]=1;
	datebook->lduration=0;
	datebook->field[8]=5;
	datebook->field[9]=0;
	datebook->field[10]=6;
	datebook->luntimed=0;
	datebook->field[11]=6;
	datebook->lprivate=0;
	datebook->field[12]=1;
	datebook->lcategory=0;
	datebook->field[13]=6;
	datebook->lalarmset=0;
	datebook->field[14]=1;
	datebook->lalarmadv=-1;
	datebook->field[15]=1;
	datebook->lalarmadvtype=0;
	datebook->field[16]=8;
	datebook->repeat.lrepeat=0;
}

