/* 

rdflexdb.c: Flexible format text reader.
Copyright (c) 2000 Narayan Natarajan

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

-----------------------------------------------
RCS stuff:
-----------------------------------------------



Author : Narayan Natarajan AKA $Author: venkman $ 
Date   : $Date: 2000/10/08 19:11:51 $ 
Version: $Revision: 1.12 $ 




rdflexdb.c : Reads a "flexible" text database. Has two main functions 
			parsefield and parse records. The first is used to parse 
			the first record in the text file. This contains the field 
			definitions and delimiters for each field. Next the parserec 
			is used to read each record and convert it into the data format 
			that convdb was initially hard coded with.  


The field defs are in form of a percent sign (%) and an alphabetic 
character (a-z, A-Z case sensitive).  


see values.h for descriptors.  


DEFAULTS: 
year=current year, duration=1hr, repeat=none, start_min=0 


DEFINITELY REQUIRED FIELDS: 
month	date	start_hour	event 




so figure all the other stuff out and create a tab delimited string 
for the convdb to process. Its less confusing this way, until convdb 
is rewritten.  




*/ 





extern timeadjust;


static char *months[] = { "january", "february", "march", "april", "may", "june", "july", "august", "september","october" , "november", "december" }; 




#include "values.h" 




int fmtcnt=0; 
int multiline=0; 
 /*these are going to contain the input field separators for every field 
 in the record , 100 separators each of 10 characters is allowed 


 recorddef[].field --- holds the separator 
 if it does then the variable: 
 recorddef[].val   --- holds zero 
 Otherwise if recorddef[].val is a character 
 then it indicates the type of field available at that location 
 in the string 
 e.g.: 
 asldfjdslkf9:20werjof 


 recorddef[0].field="asldfjdslkf" 
 recorddef[0].val = 0 


 recorddef[1].field="" 
 recorddef[1].val = 'h' 


 recorddef[2].field=":" 
 recorddef[2].val = 0 


 recorddef[3].field="" 
 recorddef[3].val = 'i' 


 recorddef[4].field="werjof" 
 recorddef[4].val =  0 


 */ 
Recdef recorddef[100]; 


void parse_fields(fpin) 
FILE *fpin; 
{ 


	int i,j; 
	char fielddef[BUFSIZ]; 


#ifdef DEBUG 
	fprintf( stderr, "Parse field\n"); 
#endif 


/* the first line is going to define what exactly resides in each record 
so read the first line 
omit comment lines 
i.e. if # is first character then read next line*/ 
	fielddef[0]='\0'; 
	do{ 
	fgets(fielddef , BUFSIZ, fpin ); 
	}while (fielddef[0] == '#' ); 


#ifdef DEBUG 
	puts(fielddef); 
#endif 


	i=j=0; 
#ifdef DEBUG 
	printf("len %d\n", strlen(fielddef)); 
#endif 
	for (i=0; i< strlen(fielddef) ;i++) 
	{ 
		/* look for a % sign and keep eating field separator */ 


		if ( fielddef[i] != '%' ) 
		{ 
		/*add field to the format string */ 
			recorddef[fmtcnt].field[j] = fielddef[i]; 
			j++; 
			recorddef[fmtcnt].val = 0; 
		} 
		else 
		{ 
			if (j != 0 ) 
			{ 
				recorddef[fmtcnt].field[j] = '\0'; 
#ifdef DEBUG 
				printf("Junk:%s:\n",recorddef[fmtcnt].field); 
#endif 
				j=0; 
				fmtcnt++; 
			} 
			recorddef[fmtcnt].val = fielddef[i+1]; 
			if ( recorddef[fmtcnt].val == MULTILINE ) 
			{ 
				/* ok, the multiline field has been set, so 
				set the variable and don't include it in the fields list 
				why? because we can't deal with it as a field, it has to be 
				read separately 


				knock off two fmtcnt's because the field separator 
				needs to be ignored as well 


				also hopefully multiline flag is the last flag, 
				it sounds safer that way...  
				*/ 
				multiline=1; 
				fmtcnt-=2; 
			} 
			fmtcnt++; 
#ifdef DEBUG 
				printf("fmtcnt:%d:\n",fmtcnt); 
#endif 
			i++; 


		} 
	} 


#ifdef DEBUG 
	puts("Fields found:"); 
	for ( i = 0 ; i < fmtcnt ; i++) 
	{ 
		switch( recorddef[i].val ) 
		{ 
			case YEAR: 
			puts("YEAR"); break; 
			case MONTH: 
			puts("MONTH");break; 
			case DATE_IN_MTH: 
			puts("DATE_IN_MTH");break; 
			case START_HOUR: 
			puts("START_HOUR");break; 
			case START_MIN: 
			puts("START_MIN");break; 
			case END_HOUR: 
			puts("END_HOUR");break; 
			case END_MIN: 
			puts("END_MIN");break; 
			case EVENT: 
			puts("EVENT");break; 
			case DURATION: 
			puts("DURATION");break; 
			case REPEAT: 
			puts("REPEAT");break; 
			case ST_TIME_FMT: 
			puts("ST_TIME_FMT");break; 
			case END_TIME_FMT: 
			puts("END_TIME_FMT");break; 
			case PRIVATE: 
			puts("PRIVATE");break; 
			case UNTIMED: 
			puts("UNTIMED");break; 




			default: 
				printf("-|%s|-",recorddef[i].field); 
		} 
	} 
	if (multiline) puts("MULTILINE"); 
#endif 


} 




void parserec(input,recorddef,output, daterecord ) 
Recdef *recorddef; 
char *input, *output; 
Datebook_entry *daterecord; 
{ 
	char fixed[BUFSIZ]; 
	char temp[10], *ptr, *fix1, *fix2, temp1, temp2; 
	int i, len; 
	time_t itime, ytime; 
	struct tm ptime, *ytm; 
	/*record definitions */ 
	int year; 
	char month[10]; 
	char  date[10]; 
	int duration,st_hour, st_min; 
	int end_hour, end_min; 
	int st_ampm, end_ampm; 
	char repeat[10]; 
	char event[BUFSIZ], note[4000]; 
	int private, untimed; 


#ifdef DEBUG 
	fprintf( stderr, "Parse Records\n"); 
#endif 


/* reset some variables */ 
	fixed[0]='\0'; 
	duration=1; 
	st_hour=end_hour=-1; 
	st_min=end_min=-1; 
	st_ampm=end_ampm=-1; 
	strcpy(repeat, "none"); 
	event[0]='\0'; 
	note[0]='\0'; 
	month[0]='\0'; 
	date[0]='\0'; 
	private = untimed =0; /* boolean flags, 0 is false */ 


	/* find out what year it is if year field is missing */ 


	time (&ytime); 
	ytm = localtime(&ytime); 
	year=ytm->tm_year ; 


	fix1=input; 


#ifdef DEBUG 
	puts(fix1); 
	fprintf( stdout, "Parse rec initted\n"); 
	fprintf( stdout, "%d fmtcnt\n", fmtcnt); 


#endif 
	/* parse the input and substitute various field delimiters 
	as tabs for ease for strtok to run 
	notes: 
		if first few charactars do not form a field but a delimiter 
		it will not be given a tab, it will just be removed 


	*/ 
	for(i  = 0 ; i < fmtcnt ; i ++) 
	{ 
		if ( recorddef[i].val != 0 ) continue; 
		/* point fix2 at start of field string */ 
		fix2 = (char *)strstr( fix1 ,recorddef[i].field); 
		temp1 = fix2[0]; 
		fix2[0] = '\0'; 


#ifdef DEBUG 
		puts(fix1); 
#endif 
		strcat(fixed, fix1 ); 
		strcat(fixed,"	"); 
		len = strlen ( recorddef[i].field ); 
		fix2[0] = temp1; 
/*		printf("jump %d %s\n", len, fix1); */ 
		fix1 = fix2 + len ; 
	} 
	strcat(fixed, fix1 ); 


#ifdef DEBUG 
	puts( "String fixed w tabs:"); 
	puts(""); 
	puts(fixed); 
	puts(""); 
#endif 


	for ( i=0 ; i < fmtcnt ; i++) 
	{ 
		if ( recorddef[i].val == 0 ) continue; 
#ifdef DEBUG1 
			printf("%d value : %c\n",i,recorddef[i].val); 
			printf("%d field: -|%s|-\n",i,recorddef[i+1].field); 
#endif 
		if ( i == 0 ) 
		{ 
			ptr=(char *) strtok( fixed, "	"); 
			if (recorddef[0].val != 0 ) 
				i=0; 
			else 
				i=1; 
		} 
		else 
			ptr=(char *) strtok( NULL, "	"); 


/* in the first cycle this should not be zero because the value has 
already been checked */ 


		if ( recorddef[i].val == 0 ) continue; 




/*			temp[0]=recorddef[i+1].field; 
			temp[1]='\0'; 
*/ 
		/* do this only after one cycle because strtok will jump 
		field */ 




		switch( recorddef[i].val ) 
		{ 
			case YEAR: 
#ifdef DEBUG 
				puts("YEAR"); 
#endif 
				year=atoi(ptr); 
#ifdef NT 
				year = (year < 100 ) ? 1900+year: year; 
#else /*WIN 95/98 */ 
				year = (year > 100 ) ? year - 1900: year; 
#endif /*NT*/ 
				break; 


			case MONTH: 
#ifdef DEBUG 
				puts("MONTH"); 
#endif 
				strcpy(month, ptr); 
				break; 


			case DATE_IN_MTH: 
#ifdef DEBUG 
				puts("DATE_IN_MTH"); 
#endif 


				strcpy(date, ptr); 
				break; 


			case START_HOUR: 
#ifdef DEBUG 
				puts("START_HOUR"); 
#endif 
				st_hour=atoi(ptr); 
				break; 


			case START_MIN: 
#ifdef DEBUG 
				puts("START_MIN"); 
#endif 
				st_min=atoi(ptr); 
				/* intialize end min to be st min, in case no end 
				time is defined */ 
				end_min = (end_min == -1 )? st_min : end_min; 
				break; 


			case END_HOUR: 
#ifdef DEBUG 
				puts("END_HOUR"); 
#endif 
				end_hour=atoi(ptr); 
				break; 
			case END_MIN: 
#ifdef DEBUG 
				puts("END_MIN"); 
#endif 
				end_min=atoi(ptr); 
				break; 
			case EVENT: 
#ifdef DEBUG 
				puts("EVENT"); 
#endif 
				strcpy(event, ptr); 
				break; 
			case DURATION: 
#ifdef DEBUG 
				puts("DURATION"); 
#endif 
				duration=atoi(ptr); 
				break; 
			case REPEAT: 
#ifdef DEBUG 
				puts("REPEAT"); 
#endif 
				strcpy(repeat, ptr); 
				break; 
			case ST_TIME_FMT: 
#ifdef DEBUG 
				puts("ST_TIME_FMT"); 
#endif 
				if( ptr[0]=='p' || ptr[0] == 'P' ) 
					st_ampm=12; 
				else 
					st_ampm=0; 
				break; 


			case END_TIME_FMT: 
#ifdef DEBUG 
				puts("END_TIME_FMT"); 
#endif 
				if( ptr[0]=='p' || ptr[0] == 'P' ) 
					end_ampm=12; 
				else 
					end_ampm=0; 
				break; 


			case PRIVATE: 
			private=-1; 
			switch ( ptr[0]) 
			{ 
				case 'y': private=1; break; 
				case 'Y': private=1; break; 
				case 'n': private=0; break; 
				case 'N': private=0; break; 
			} 


			if(private == -1 ) 
				private  = atoi (ptr); 
#ifdef DEBUG 
				puts("PRIVATE"); 
				printf("private %d\n", private); 
#endif 
				break; 
			case UNTIMED: 
			untimed=-1; 
			switch ( ptr[0]) 
			{ 
				case 'y': untimed=1; break; 
				case 'Y': untimed=1; break; 
				case 'n': untimed=0; break; 
				case 'N': untimed=0; break; 
			} 


			if(untimed == -1 ) 
				untimed  = atoi (ptr); 
/* if the untimed flag is set then start hour is set to zero 
so that the date computation forms a valid value 
any value would do but why bother 
*/ 
			if(untimed == 1 ) st_hour = 0; 


#ifdef DEBUG 
				puts("UNTIMED"); 
				printf("untimed %d\n", untimed); 
#endif 
				break; 
	
		} 
/*		i++;*/ 
#ifdef DEBUG 
		puts(ptr); 
		puts("----"); 
#endif 
	} 
	/*at the end of this figure out what's missing */ 
	/* starting from the top */ 

	/*check the min see if -ve, if so reset */ 
	st_min = (st_min == -1)? 0 : st_min; 
	end_min = (end_min == -1)? st_min : end_min; 

	/* add ampm flag to start hour if hours are less 
	than 12 to make a 24 hour format*/ 

	/* if st_hour is 12 and ampm is set then make st_hour zero
	i.e. its 1201 in the midnight so its 0001 in the night. */

/* if st_ampm = -1 then ampm is not set so leave the st_hour alone */
	if ( st_ampm == 0 )
	{
		/* check if its 12:XX AM and fix it to 00:XX */
		st_hour= ( st_hour == 12 )? 0 : st_hour ;
	}
	else
	{
		if ( st_ampm == 12 )
		{
			if ( st_hour != 12 )
			{
				/*time is between 1-11*/
				st_hour +=st_ampm;
			}
			else
			{
				if (st_min == 0 )
				{
				fprintf(stderr, "The time specified was illegal\n");
				fprintf(stderr, "Record Details: \n");
				fprintf(stderr, "%s\n", input );
				fprintf(stderr, "WARNING: ASSUMING THAT THE TIME IS 12:00 NOON!!\n\n");
				}
			}
		}
	}

	if ( end_hour == -1 )
	{
		end_hour = st_hour + duration;
		if ( end_hour > 23 )
		{
			end_min  = 59;
			end_hour = 23;
		}
	}

	if (end_ampm == 0 )
	{
		/* check if its 12:XX AM and fix it to 00:XX */
		end_hour= ( end_hour == 12 )? 0 : end_hour ;
	}
	else
	{
		if ( end_ampm == 12 )
		{
			if ( end_hour != 12 )
			{
				/*time is between 1-11*/
				end_hour +=end_ampm;
			}
			else
			{
				if (end_min == 0 )
				{
				fprintf(stderr, "The time specified was illegal\n");
				fprintf(stderr, "Record Details: \n");
				fprintf(stderr, "%s\n", input );
				fprintf(stderr, "WARNING: ASSUMING THAT THE TIME IS 12:00 NOON!!\n\n");
				}
			}

		}
		else
		end_hour = ( end_hour == 12 ) ? 0: end_hour;
	}





	if (year == 0 ) 
	{ 
		time(&itime); 
		ptime = *localtime(&itime); 
#ifdef NT 
		year=ptime.tm_year + 1900; 
#else /* WIN 95/98 */ 
		year=ptime.tm_year; 
#endif /*NT*/ 
	} 


/* send this info to the data loading function 
I know nothing is optimised, but who cares...  

>:| I do! dammit!

*/ 

#ifdef DEBUG
	printf("%d\t%d\t%d\t%s\t%s\t%d\t%d\t%d\t%d\t%s\t%s\n", 
	private, untimed, year,month, date, st_hour, st_min, end_hour, end_min, repeat, event); 


#endif /*DEBUG*/
	load_entry( daterecord, 
	private, untimed, year,month, date, st_hour, st_min, end_hour, end_min, 
	repeat, event); 




	/*print all this into the fixed string */ 
	sprintf(fixed, "%d\t%d\t%d\t%s\t%s\t%d\t%d\t%d\t%d\t%s\t%s\n", 
	private, untimed, year,month, date, st_hour, st_min, end_hour, end_min, repeat, event); 
#ifdef DEBUG 
	puts(fixed); 
#endif 
	strcpy(output , fixed); 


} 


load_entry(daterecord, 
private, untimed, year,month, date, st_hour, st_min, end_hour, end_min, 
repeat, event) 
Datebook_entry *daterecord; 
/*record definitions */ 
int year; 
char month[10]; 
char  date[10]; 
int st_hour, st_min; 
int end_hour, end_min; 
char repeat[10]; 
char event[BUFSIZ]; 
int private, untimed; 
{ 
	struct tm starttime, endtime; 
	int i; 
	char *ptr, *retptr; 
	time_t t_start, t_end; 
	char repeat_type[10]; 
	struct tm *temptime; 

#ifdef DEBUG
	puts ("Load Entry initted"); 
	/* zero everything */ 
#endif /*DEBUG*/

	endtime.tm_year = starttime.tm_year =0; 
	endtime.tm_mon = starttime.tm_mon =0; 
	endtime.tm_mday = starttime.tm_mday =0; 
	starttime.tm_sec = starttime.tm_hour =0; 
	starttime.tm_min =0; 
	endtime.tm_sec = endtime.tm_hour =0; 
	endtime.tm_min =0; 

	strcpy(repeat_type, repeat);
/* new code for equating all the above stuff */ 
	daterecord->lprivate=private; 
	daterecord->luntimed=untimed; 
	endtime.tm_year = starttime.tm_year = year; 
/* month may be in character format (Jan, feb , march etc) or numerical */ 


	for ( i = 0; i < 12 ; i++) 
	if ( !strnicmp( months[i], month, strlen( month ) ) ) 
		break; 
	if ( i == 12 ) 
	{ 
		endtime.tm_mon = starttime.tm_mon = atoi( month ) -1; 


	} 
	else 
		endtime.tm_mon = starttime.tm_mon = i; 


	endtime.tm_mday = starttime.tm_mday = atoi(date); 
	starttime.tm_wday = starttime.tm_yday = 0; 
	endtime.tm_wday = endtime.tm_yday = 0; 
	starttime.tm_hour = st_hour; 
	starttime.tm_min  = st_min; 
	endtime.tm_hour   = end_hour; 
	endtime.tm_min    = end_min; 


	/*read the event info string strip return character */ 
	/* also its GOTTA be the last character, anywhere else
	it would spill into the note field... see? */
	retptr=(char*) strchr(event , '\r');
	if (retptr) *retptr='\0';
	retptr=(char*) strchr(event , '\n');
	if (retptr) *retptr='\0';


	strcpy(daterecord->description.string, event); 
	daterecord->description.byte = strlen(event); 

	/* also string of length greater than 256 is invalid so basically 
	trim it to 255 */ 


	if ( strlen (daterecord->description.string) > 255 ) 
	{ 
		daterecord->description.string[254] = '\0'; 
		daterecord->description.byte = 254; 
		/* have to debug a bit more 
		string cutoff seems to allow only 254 not 255 
		*/ 
		fprintf(stderr,"---------------\n"); 
		fprintf(stderr,"Warning: Event description is longer than 255 characters.\n"); 
		fprintf(stderr,"Action: trimmed\n"); 
		fprintf(stderr,"Result:\n%s\n", 
		daterecord->description.string ); 
		fprintf(stderr,"---------------\n"); 
	} 


	/* convert month if in text 


	for ( i = 0; i < 12 ; i++) 
	if ( !strnicmp( months[i], month, strlen( month ) ) ) 
	{ 
	puts("matched"); 
		break; 
	} 


	if ( i == 12 ) 
		endtime.tm_mon = starttime.tm_mon = atoi( month) -1; 


	else 
		endtime.tm_mon = starttime.tm_mon = i; 
 */ 
	endtime.tm_isdst = starttime.tm_isdst =  0; 


#ifdef DEBUG 
	printf("%d/%d/%d %d:%d:%d wday %d yday %d dst %d \n", 
		starttime.tm_year,starttime.tm_mon, starttime.tm_mday, 
		starttime.tm_hour, starttime.tm_min, starttime.tm_sec, 
		starttime.tm_wday, starttime.tm_yday, starttime.tm_isdst); 
#endif /*DEBUG*/ 


	t_start= mktime( &starttime ) + timeadjust; 


#ifdef DEBUG 
	printf("t_start %s %d\n", ctime(&t_start), starttime.tm_min); 
	printf("%d/%d/%d %d:%d:%d wday %d yday %d dst %d \n", 
		starttime.tm_year,starttime.tm_mon, starttime.tm_mday, 
		starttime.tm_hour, starttime.tm_min, starttime.tm_sec, 
		starttime.tm_wday, starttime.tm_yday, starttime.tm_isdst); 
#endif /*DEBUG*/ 
	t_end =  mktime( &endtime ) + timeadjust; 
	t_start = t_start - (starttime.tm_isdst *3600); 
	t_end = t_end - (endtime.tm_isdst *3600); 


	daterecord->start_time = t_start; 
	daterecord->end_time = t_end; 


#ifdef DEBUG 
	printf("start: %s\n", ctime(&t_start)); 
	printf("end  : %s\n", ctime(&t_end)); 
#endif /*DEBUG*/ 




	if ( !stricmp (repeat_type,  DAY_OF_MONTH) ) 
	{ 
		/* set the repeat value to 0xffff0000 to indicate the 
			entry repeats */ 


		daterecord->repeat.lrepeat=(unsigned long) 0xffff0000; 
/* 
:ab dbk dbook.datebook[counter].repeat 
*/ 
		daterecord->repeat.lunknown= 0x000B0001; 
		strcpy (daterecord->repeat.creptype , "CDayOfMonth" ); 
		daterecord->repeat.lreptype = 3; 
		daterecord->repeat.lfreq = 12; 
/* 
Copied this right from the dat file, 
happens to be: 
Wed Dec 31 00:00:00 2031 
*/ 
		daterecord->repeat.lenddate = 0x749d5e80; 
/* a zero follows this */ 
		daterecord->repeat.lcustom[0] = 0; 


		temptime = localtime ( &t_start ); 
#ifdef DEBUG 
		printf("week day %d: week of month %d %d\n reptype %d\n", temptime->tm_wday, (temptime->tm_mday - 1)  / 7 , temptime->tm_year, daterecord->repeat.lreptype); 
#endif /*DEBUG*/ 
/*			strftime( tempstr, 99, "%w" , &temptime ); 
		printf(" day of week %s \n", tempstr ); */ 


		daterecord->repeat.lcustom[1] = temptime->tm_wday; 
		daterecord->repeat.lcustom[2] = ( temptime->tm_mday - 1) / 7; 


	} 
	else 
	if ( !stricmp (repeat_type,  DATE_OF_YEAR) ) 
	{ 
		/* set the repeat value to 0xffff0000 to indicate the 
			entry repeats */ 


		daterecord->repeat.lrepeat=(unsigned long) 0xffff0000; 
/* 
:ab dbk dbook.datebook[counter].repeat 
*/ 
		daterecord->repeat.lunknown= 0x000B0001; 
		strcpy (daterecord->repeat.creptype , "CDateOfYear" ); 
		daterecord->repeat.lreptype = 5; 
		daterecord->repeat.lfreq = 1; 
/* 
Copied this right from the dat file, 
happens to be: 
Wed Dec 31 00:00:00 2031 
*/ 
		daterecord->repeat.lenddate = 0x749d5e80; 
/* a zero follows this */ 
		daterecord->repeat.lcustom[0] = 0; 


		temptime = localtime ( &t_start ); 
#ifdef DEBUG 
		printf("week day %d: date %d %d\n reptype %d\n", temptime->tm_wday, temptime->tm_mday / 7 , temptime->tm_year, daterecord->repeat.lreptype); 
#endif /*DEBUG*/ 
/*			strftime( tempstr, 99, "%w" , &temptime ); 
		printf(" day of week %s \n", tempstr ); */ 


		daterecord->repeat.lcustom[1] = temptime->tm_mday; 
		daterecord->repeat.lcustom[2] = temptime->tm_mon ; 


	} 
	else 
		daterecord->repeat.lrepeat= 0; 




} 
